<!--    Authors         :   Carlos C. Corrada-Bravo
                            David J. Ortiz-Rivera

        Organization    :   Centro de Desarrollo y Consultoria Computacional
        Project         :   OPASO Material Registry  
        File            :   manage-personnel
        Description     :   Manage laboratory personnel -->

<!-- verify session -->
<?php require_once("header.php");

    /* start user session */
    if(!isset($_SESSION)){
        session_start();
    }
    if(!($_SESSION["access_level"] === "admin") and !($_SESSION["access_level"] === "investigator")){
        header("Location: /menu");
        die();
    }
?>

<!doctype html>
<html lang="en">
    <!-- encoding/imports/styling -->
    <head>
        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title class="title">Manage Personnel</title>
        <link rel="stylesheet" href="https://stackpath.bootstrapcdn.com/bootstrap/4.2.1/css/bootstrap.min.css" integrity="sha384-GJzZqFGwb1QTTN6wy59ffF1BuGJpLSa9DkKMp0DgiMDm4iYMj70gZWKYbI706tWS" crossorigin="anonymous">
        <link href="https://fonts.googleapis.com/icon?family=Material+Icons" rel="stylesheet">
        <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.6.3/css/all.css" integrity="sha384-UHRtZLI+pbxtHCWp1t77Bi1L4ZtiqrqD80Kn4Z8NTSRyMA2Fd33n5dQ8lWUE00s/" crossorigin="anonymous">
        <link rel="stylesheet" href="./style/main.css">
        <link rel="stylesheet" href="/style/manage.css">
        <link rel="stylesheet" href="/style/add.css">
    </head>

    <!-- content -->
    <body>
        <!-- forms -->
        <div class="form-shader close-form">
            <!-- access form -->
            <div class="main-form close-form">
                <div class="form-wrapper">
                    <div class="form-header-wrapper">
                        <h5 class="form-header main-header"></h5>
                        <div class="close-icon"><i class="close-form material-icons">close</i></div>
                    </div>
                    <div class="form">
                        <div class="form-body">
                            <h6 class="form-header person"></h6>
                            <div class="select-wrapper">
                                <label class="select-label" for="number">Laboratory</label>
                                <select class="laboratories"></select>
                            </div>
                            <div class="select-wrapper">
                                <label class="select-label" for="number">Access Level</label>
                                <select class=access_level>
                                    <option value="investigator">Investigator</option>
                                    <option value="technician">Technician</option>
                                    <option value="none">None</option>
                                </select>
                            </div>
                        </div>
                        <div class="form-actions">
                            <button class="btn btn-primary action-btn submit" id="main-submit">Submit</button>
                        </div>
                    </div>
                </div>
            </div>

            <!-- add form -->
            <div class="add-form close-form">
                <div class="add-form-wrapper">
                    <div class="add-form-header-wrapper">
                        <h5 class="form-header">Add Personnel</h5>
                        <div class="close-icon"><i class="close-form material-icons">close</i></div>
                    </div>
                    <div class="add-form-body-wrapper">
                        <div class="add-form-body">
                            <div class="details material">
                                <div class="li-wrapper medium-field">
                                    <label class="form-label" for="name">Full Name</label>
                                    <input id="person_name" class="required ap-field" placeholder="Full name" type="text" name="name">
                                    <div class="invalid-feedback person_name">Please provide a valid name</div>
                                </div>
                                <div class="li-append">
                                    <div class="li-wrapper medium-field">
                                        <label class="form-label" for="email">Email</label>
                                        <input id="email" class="required ap-field" placeholder="someone@upr.edu" type="text" name="email">
                                        <div class="invalid-feedback email">Please provide a valid email</div>
                                    </div>
                                    <div class="li-wrapper small-field margin-left phone-number">
                                        <label class="form-label" for="number">Contact Number</label>
                                        <input id="phone_number" class="ap-field" type="text" name="number" placeholder="Phone number" oninput="this.value = this.value.replace(/[^0-9.]/g,'').replace(/(\..*)\./g,'$1');">
                                    </div>                               
                                </div>
                            </div>
                        </div>
                        <div class="add-form-actions">
                            <button class="btn btn-primary action-btn submit" id="add-submit">Submit</button>
                        </div>
                    </div>
                </div>
            </div> 
        </div>

        <!-- alerts -->
        <div class="alerts">
            <div class="alert">
                <div class="message-icon-wrapper">
                    <i class="message-icon success-icon material-icons">done</i>
                    <i class="message-icon error-icon failed-icon warning-icon material-icons">close</i>
                </div>
                <p class="alert-status"></p>
                <p class="alert-message"></p>
            </div>
        </div>

        <div class="content">
            <!-- navbar -->
            <?php require_once("navbar.php")?>
            <div class="main">
                <div class="header-wrapper">
                    <h5 class="header">Manage Personnel</h5>

                    <!-- processing -->
                    <div class="processing">
                        <p class="processing-text">Fetching personnel...</p>
                        <div class="loader"></div>
                    </div>
                </div>
                <div class="main-wrapper">
                    <!-- search form -->
                    <div class="float-right">
                        <div class="transaction btn btn-light" id="add">
                            <i class="fas fa-plus add-personnel-icon"></i>
                            <p class="add-text">Add</p>
                        </div>

                        <div class="search-form-wrapper">
                            <input class="search-form" placeholder="Search for personnel" type="text" name="search-form">
                            <button class="clear-btn search-form-submit">
                                <i class="material-icons clear-icon">close</i>
                            </button>
                        </div>
                    </div>

                    <!-- personnel -->
                    <h6 class="header sub-header">Personnel List</h6>
                    <div class="table-wrapper content-wrapper personnel">
                        <table class="table">
                            <thead class="table-header">
                                <tr>
                                    <th value="person">Person</th>
                                    <th >Authorized</th>
                                    <th >Access Level</th>
                                    <th >Actions</th>
                                </tr>
                            </thead>
                            <tbody></tbody>
                        </table>
                    </div>

                    <h6 class="header table-total"></h6>
                </div>
            </div>

            <!-- footer -->
            <?php require_once("footer.php")?>
        </div>  

        <!-- scripts -->
        <script src="https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js"></script>
        <script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.6/umd/popper.min.js" integrity="sha384-wHAiFfRlMFy6i5SRaxvfOCifBUQy1xHdJ/yoi7FRNXMRBu5WHdZYu1hA6ZOblgut" crossorigin="anonymous"></script>
        <script src="https://stackpath.bootstrapcdn.com/bootstrap/4.2.1/js/bootstrap.min.js" integrity="sha384-B0UglyR+jN6CkvvICOB2joaf5I4l3gm9GU6Hc1og6Ls7i6U/mkkaduKaBhlAXv9k" crossorigin="anonymous"></script>
        <script type="text/javascript" src="/scripts/main.js"></script>
        <script type="text/javascript" src="/scripts/manage-personnel.js"></script>
    </body>
</html>